/* code03e8.c - EvtProcessSoftKeyStroke handler
   Copyright  1999 Ben Darnell
   This file is a part of LinkMaster,
   which is distributed under the GNU GPL.
   Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/

#include <Pilot.h>
#include <FeatureMgr.h>
/*#include <SysEvtMgr.h>*/
/* gcc doesnt like that header file, so here's the decl i need: */
Err EvtFlushNextPenStroke() SYS_TRAP(sysTrapEvtFlushNextPenStroke);
#include <SysEvtPrv.h>
#include "linkhistoryhack.h"
#include "linkhistoryhackRsc.h"
#include "linkaware.h"

static Boolean PopupList(Boolean bookmarks);

// This is the code that will be called whenever ANY app calls EvtGetEvent,
// and myhack is enabled in HackMaster. HackMaster is not smart about the
// content of this file; it will simply jump to the first executable statement
// in this file. If you need helper functions, prototype them above, and
// define them AFTER the trap patch.
Err EvtProcessSoftKeyStroke(PointType* start, PointType* end)
{
   DWord               temp;
   Err              (*oldTrap)(PointType*, PointType*);

   PenBtnInfoPtr btninfo;
   UInt numbuttons=0, i;
   UInt startbutton=0, endbutton=0, action=0;

   // get the address of the old EvtGetEvent() from HackMaster
   FtrGet(MYCRID, 0x3e8, &temp);
   // set the function pointer to the old trap
   oldTrap = (Err (*)(PointType*, PointType*)) temp;

   btninfo=EvtGetPenBtnList(&numbuttons);
   for (i=0; i<numbuttons; i++) {
      if (RctPtInRectangle(start->x, start->y, &btninfo[i].boundsR))
	 startbutton=btninfo[i].asciiCode;
      if (RctPtInRectangle(end->x, end->y, &btninfo[i].boundsR))
	 endbutton=btninfo[i].asciiCode;
   };

   if (startbutton!=endbutton) {
      MyPrefsType prefs;
      Word prefssize=sizeof(prefs);

      if ((PrefGetAppPreferences('LMHH', 0, &prefs, &prefssize, true)
	   ==noPreferenceFound)||(prefssize<sizeof(prefs))) {
	 prefs.backstart=findChr;
	 prefs.backend=menuChr;
	 prefs.fwdstart=menuChr;
	 prefs.fwdend=findChr;
	 prefs.liststart=menuChr;
	 prefs.listend=launchChr;
      }
      if ((startbutton==prefs.backstart)&&(endbutton==prefs.backend))
	 action=linkAppLaunchHistoryBack;
      if ((startbutton==prefs.fwdstart)&&(endbutton==prefs.fwdend))
	 action=linkAppLaunchHistoryForward;
      if ((startbutton==prefs.liststart)&&(endbutton==prefs.listend)) {
	 Boolean bookmarks=false;
	 EvtFlushNextPenStroke();
	 while (1) {
	    if (PopupList(bookmarks))
	       return 0;
	    bookmarks=!bookmarks;
	 };
      };
   };

   if (action) {
      Err err;
      DmSearchStateType searchstate;
      UInt cardno;
      LocalID dbid;
      DWord result;

      err=DmGetNextDatabaseByTypeCreator(true, &searchstate, 'appl', 'Link',
					 true, &cardno, &dbid);
      if (!err) 
	 SysAppLaunch(cardno, dbid, 0, action, 0, &result);
   };
   
   return oldTrap(start, end);
}

static void DrawList(UInt itemnum, RectanglePtr bounds, DmOpenRef db)
{
   Word index;
   VoidHand hrec;
   FontID oldfont;

   index=DmNumRecords(db)-itemnum;
   if (index==0) {
      Char text0[]="History";
      Char text1[]="Bookmarks";
      CharPtr text;

      oldfont=FntSetFont(boldFont);
      if (oldfont==stdFont)
	 text=text0;
      else
	 text=text1;
      WinDrawChars(text, StrLen(text), bounds->topLeft.x, bounds->topLeft.y);
      FntSetFont(oldfont);
      return;
   } else {
      index--;

      hrec=DmQueryRecord(db, index);
      if (hrec) {
	 Ptr prec;
	 CharPtr text;
	 
	 prec=MemHandleLock(hrec);
	 text=prec+64;
	 oldfont=FntSetFont(stdFont);
	 WinDrawChars(text, StrLen(text),
		      bounds->topLeft.x, bounds->topLeft.y);
	 FntSetFont(oldfont);
	 MemHandleUnlock(hrec);
      };
   };
}

/* returns true if an item was selected, false to switch db's */
static Boolean PopupList(Boolean bookmarks)
{
   ListType list;
   DmOpenRef db;
   short sel;
   WinHandle win;

   win=WinGetDrawWindow();
   WinSetDrawWindow(WinGetDisplayWindow());

   if (bookmarks)
      db=DmOpenDatabaseByTypeCreator('DATA', 'Link', dmModeReadOnly);
   else 
      db=DmOpenDatabaseByTypeCreator('HIST', 'Link', dmModeReadOnly);
   if (!db) return true;
   if (!DmNumRecords(db)) {
      DmCloseDatabase(db);
      return true;
   };

   MemSet(&list, sizeof(list), 0);
   list.bounds.topLeft.x=0;
   list.bounds.topLeft.y=3;
   list.bounds.extent.x=80;
   list.bounds.extent.y=154;
   /* sneak some extra data to the callback function this way */
   if (bookmarks)
      list.font=boldFont;
   else
      list.font=stdFont;
   list.numItems=DmNumRecords(db)+1;
   if (list.numItems>14)
      list.topItem=list.numItems-14;
   else
      list.topItem=0;
   list.attr.usable=true;
   list.drawItemsCallback=(ListDrawDataFuncPtr)DrawList;
   list.itemsText=(CharPtr*)db;
   sel=LstPopupList(&list);

   if (sel>=0) {
      /* this code snipped from linkaware.c */
      LinkInfoPtr params;
      DmSearchStateType searchstate;
      UInt cardno;
      LocalID dbid;
      Err err;
      DWord result;
      VoidHand hrec;
      Ptr prec;
      Word index;

      index=DmNumRecords(db)-sel;
      if (index==0) {
	 DmCloseDatabase(db);
	 return false;
      };
      index--;

      hrec=DmQueryRecord(db, index);
      if (!hrec) goto exit;
      prec=MemHandleLock(hrec);
      
      params=MemPtrNew(64);
      MemPtrSetOwner(params, 0);
      MemMove(params, prec, 64);

      MemHandleUnlock(hrec);
      
      err=DmGetNextDatabaseByTypeCreator(true, &searchstate,
					 'appl', params->creator,
					 true, &cardno, &dbid);
      if (err)
	 err=DmGetNextDatabaseByTypeCreator(true, &searchstate,
					    'PROX', params->creator,
					    true, &cardno, &dbid);
      if (err) {MemPtrFree(params); goto exit;};
      SysAppLaunch(cardno, dbid, 0,
		   linkAppLaunchFollowLinkSubCall, (Ptr)params, &result);
   };
 exit:
   DmCloseDatabase(db);
   WinSetDrawWindow(win);
   return true;
}
