/* Main code for Proxy Address */

#include <Pilot.h>
#include "callback.h"
#include "AddressDB.h"

#include "linkaware.h"
#include "proxyaddrRsc.h"

static DmOpenRef AddressDB;

static VoidPtr GetObjectPtr(Word id)
{
   FormPtr frm;
   frm=FrmGetActiveForm();
   return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, id));
}

static CharPtr GetName(AddrDBRecordPtr r)
{
   CharPtr s;
   if (r->fields[name]) {
      if (r->fields[firstName]) {
	 s=MemPtrNew(StrLen(r->fields[name])+StrLen(r->fields[firstName])+2);
	 StrPrintF(s, "%s %s", r->fields[firstName], r->fields[name]);
      } else {
	 s=MemPtrNew(StrLen(r->fields[name])+1);
	 StrCopy(s, r->fields[name]);
      };
   } else if (r->fields[firstName]) {
      s=MemPtrNew(StrLen(r->fields[firstName])+1);
      StrCopy(s, r->fields[firstName]);
   } else if (r->fields[company]) {
      s=MemPtrNew(StrLen(r->fields[company])+1);
      StrCopy(s, r->fields[company]);
   } else {
      s=MemPtrNew(1);
      StrCopy(s, "");
   };
   return s;
}

static void MainListDraw(UInt item, RectanglePtr bounds, CharPtr* text)
{
   AddrDBRecordType r;
   Handle handle;
   Word index;
   CharPtr s;
   
   CALLBACK_PROLOGUE;
   index=0;
   DmSeekRecordInCategory(AddressDB, &index, item, dmSeekForward,
			  dmAllCategories);
   AddrGetRecord(AddressDB, index, &r, &handle);
   s=GetName(&r);
   WinDrawChars(s, StrLen(s), bounds->topLeft.x, bounds->topLeft.y);
   MemPtrFree(s);
   MemHandleUnlock((VoidHand)handle);
   CALLBACK_EPILOGUE;
}

static void MainFormInit()
{
   ListPtr list;

   AddressDB=DmOpenDatabaseByTypeCreator('DATA', 'addr', dmModeReadOnly);
   
   list=GetObjectPtr(MainList);
   LstSetListChoices(list, NULL,
		     DmNumRecordsInCategory(AddressDB, dmAllCategories));
   LstSetDrawFunction(list, MainListDraw);
}

static Boolean MainFormHandleEvent (EventPtr e)
{
   Boolean handled = false;
   FormPtr frm;
    
   CALLBACK_PROLOGUE

      switch (e->eType) {
      case frmOpenEvent:
	 MainFormInit();
	 frm = FrmGetActiveForm();
	 FrmDrawForm(frm);
	 handled = true;
	 break;

      case menuEvent:
	 MenuEraseStatus(NULL);

	 switch(e->data.menu.itemID) {
	 }

	 handled = true;
	 break;

      case ctlSelectEvent:
	 switch(e->data.ctlSelect.controlID) {
	 case MainPublishButton: {
	    AddrDBRecordType r;
	    Word index;
	    Handle handle;
	    CharPtr s;

	    index=0;
	    DmSeekRecordInCategory(AddressDB, &index,
				   LstGetSelection(GetObjectPtr(MainList)),
				   dmSeekForward, dmAllCategories);
	    AddrGetRecord(AddressDB, index, &r, &handle);
	    s=GetName(&r);
	    LinkPublish((LinkInfoPtr)LinkSimpleNew(AddressDB,
						   index, s));
	    MemPtrFree(s);
	    MemHandleUnlock((VoidHand)handle);
	    break;
	 };

	 default:
	    break;
	 }
	 break;

      default:
	 break;
      }

   CALLBACK_EPILOGUE

      return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
   FormPtr frm;
   Word    formId;
   Boolean handled = false;

   if (e->eType == frmLoadEvent) {
      formId = e->data.frmLoad.formID;
      frm = FrmInitForm(formId);
      FrmSetActiveForm(frm);

      switch(formId) {
      case MainForm:
	 FrmSetEventHandler(frm, MainFormHandleEvent);
	 break;
      }
      handled = true;
   }

   return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(void)
{
   FrmGotoForm(MainForm);
   return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
   FrmSaveAllForms();
   FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
   Word err;
   EventType e;

   do {
      EvtGetEvent(&e, evtWaitForever);
      if (! SysHandleEvent (&e))
	 if (! MenuHandleEvent (NULL, &e, &err))
	    if (! ApplicationHandleEvent (&e))
	       FrmDispatchEvent (&e);
   } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
   Word err;

   switch (cmd) {
   case sysAppLaunchCmdNormalLaunch:
   case linkAppLaunchRequestLink:
      err = StartApplication();
      if (err) return err;
      EventLoop();
      StopApplication();
      break;

   case linkAppLaunchFollowLinkSubCall: {
      GoToParamsPtr gotoparams;
      DmSearchStateType searchstate;
      UInt cardno;
      LocalID dbid;
      Err err;

      gotoparams=LinkSimpleToGoToParams((LinkSimplePtr)cmdPBP);
      /* try address+ first */
      err=DmGetNextDatabaseByTypeCreator(true, &searchstate, 'appl', 'addp',
					 true, &cardno, &dbid);
      if (err==dmErrCantFind)
	 err=DmGetNextDatabaseByTypeCreator(true, &searchstate, 'appl', 'addr',
					    true, &cardno, &dbid);
      MemPtrSetOwner(gotoparams, 0);
      SysUIAppSwitch(cardno, dbid, sysAppLaunchCmdGoTo, (Ptr)gotoparams);
      break;
   };

   default:
      return sysErrParamErr;
   }

   return 0;
}
