/* Main code for Proxy Find */

#include <Pilot.h>
#include <CharAttr.h>
#include "callback.h"

#include "linkaware.h"
#include "proxyfindRsc.h"

typedef struct {
   DWord creator;
   UInt cardno;
   LocalID dbid;
   GoToParamsType gotoparams;
} FindLinkInfoType;
typedef FindLinkInfoType* FindLinkInfoPtr;

static Handle happlist;
static SysDBListItemType* papplist;

static void AppDrawFunc(UInt index, RectanglePtr bounds, CharPtr* data)
{
   SysDBListItemType* item;
   CALLBACK_PROLOGUE;

   item=papplist+index;
   WinDrawChars(item->name, StrLen(item->name),
		bounds->topLeft.x, bounds->topLeft.y);
   CALLBACK_EPILOGUE;
}

static void MainFormInit()
{
   FormPtr frm;
   Word appcount;
   ListPtr list;

   frm=FrmGetActiveForm();

   if (!SysCreateDataBaseList('appl', 0, &appcount, &happlist, true))
      ErrDisplay("Error getting list of applications");
   papplist=MemHandleLock((VoidHand)happlist);

   list=FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainList));
   LstSetListChoices(list, NULL, appcount);
   LstSetDrawFunction(list, AppDrawFunc);
   LstSetSelection(list, 0);
}

static void CleanUp()
{
   MemHandleUnlock((VoidHand)happlist);
   MemHandleFree((VoidHand)happlist);
}

FindLinkInfoPtr FindLinkNew(UInt cardno, LocalID dbid,
			    GoToParamsPtr gotoparams, CharPtr title)
{
   FindLinkInfoPtr linkinfo;

   if (title) {
      /* title is truncated to 32 chars or at the first \n */
      int titlelen=32;
      int tmp;

      tmp=StrLen(title);
      if (tmp<titlelen) titlelen=tmp;

      tmp=StrChr(title, '\n')-title;
      if ((tmp>0)&&(tmp<titlelen)) titlelen=tmp;

      linkinfo=MemPtrNew(65+titlelen);
      MemSet(linkinfo, 65+titlelen, 0);
      ErrFatalDisplayIf(!linkinfo, "Not enough memory");
      StrNCopy(((CharPtr)linkinfo)+64, title, titlelen);
      ((CharPtr)linkinfo)[64+titlelen]=0;
   } else {
      linkinfo=MemPtrNew(64);
      MemSet(linkinfo, 64, 0);
   };

   linkinfo->creator='LPfi';
   linkinfo->cardno=cardno;
   linkinfo->dbid=dbid;
   MemMove(&linkinfo->gotoparams, gotoparams, sizeof(GoToParamsType));

   return linkinfo;
}

/* based on thoughtstream-palmos/idea.c/SearchActivate and the sys Find dlg */
static void Publish()
{
   FormPtr frm;
   ListPtr list;
   Word selection;
   CharPtr title;
   GoToParamsType gotoparams;
   VoidHand hfindparams;
   FindParamsPtr pfindparams;
   int len;
   BytePtr convtbl;
   SystemPreferencesType sysprefs;
   FormPtr findresults;
   TablePtr tbl;
   int row, i;
   DWord dummy;
   SWord selmatch=-1;

   /* get selection */
   frm=FrmGetActiveForm();
   list=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm, MainList));
   selection=LstGetSelection(list);

   /* get title */
   title=FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
							      MainField)));

   /* init find params */
   hfindparams=MemHandleNew(sizeof(FindParamsType));
   pfindparams=(FindParamsPtr)MemHandleLock(hfindparams);
   MemSet(pfindparams, sizeof(FindParamsType), 0);
   pfindparams->newSearch=true;
   pfindparams->lineNumber=0;

   len=StrLen(title);
   if (len>maxFindStrLen)
      len=maxFindStrLen;
   StrNCopy(pfindparams->strAsTyped, title, len);
   pfindparams->strAsTyped[maxFindStrLen]=0;
   convtbl=GetCharCaselessValue();
   for (i=0; i<len; i++)
      pfindparams->strToFind[i]=convtbl[pfindparams->strAsTyped[i]];
   pfindparams->strToFind[len]=0;

   PrefGetPreferences(&sysprefs);
   if (sysprefs.hideSecretRecords)
      pfindparams->dbAccesMode=dmModeReadOnly;
   else
      pfindparams->dbAccesMode=dmModeReadOnly|dmModeShowSecret;

   SysCurAppDatabase(&pfindparams->callerAppCardNo,
		     &pfindparams->callerAppDbID);

   pfindparams->appCardNo=papplist[selection].cardNo;
   pfindparams->appDbID=papplist[selection].dbID;

   /* prepare the form */
   findresults=FrmInitForm(FindResultsDialog);
   FrmSetActiveForm(findresults);
   tbl=FrmGetObjectPtr(findresults, FrmGetObjectIndex(findresults,
						      FindResultsTable));
   TblSetColumnUsable(tbl, 0, true);
   for (row=0; row<TblGetNumberOfRows(tbl); row++) {
      TblSetRowUsable(tbl, row, false);
      TblSetRowSelectable(tbl, row, false);
   };
   FrmDrawForm(findresults);
   CtlHideControl(FrmGetObjectPtr(findresults,
				  FrmGetObjectIndex(findresults,
						    FindResultsStopButton)));
   CtlShowControl(FrmGetObjectPtr(findresults,
				  FrmGetObjectIndex(findresults,
						    FindResultsCancelButton)));
   while (EvtEventAvail()) {
      /* flush event queue before starting find */
      EventType e;
      EvtGetEvent(&e, 0);
      SysHandleEvent(&e);
   };

   /* scan selected app */
   if (SysAppLaunch(pfindparams->appCardNo, pfindparams->appDbID, 0,
		    sysAppLaunchCmdFind, (Ptr)pfindparams, &dummy)) {
      FrmAlert(SearchErrorAlert);
      goto exit;
   }

   /* get selection from user */
   if (pfindparams->numMatches<1) {
      FrmAlert(SearchNotFoundAlert);
      goto exit;
   };
   if (pfindparams->numMatches==1)
      selmatch=0;
   else {
      Boolean done=false;
      EventType event;

      while (!done) {
	 EvtGetEvent(&event, 0);
	 FrmHandleEvent(findresults, &event);
	 if (SysHandleEvent(&event))
	    continue;
	 switch (event.eType) {
	 case tblSelectEvent:
	    selmatch=event.data.tblSelect.row-1;
	    done=true;
	    break;

	 case ctlSelectEvent:
	    if (event.data.ctlSelect.controlID==FindResultsCancelButton) {
	       selmatch=-1;
	       done=true;
	    };
	    break;

	 case appStopEvent:
	    EvtAddEventToQueue(&event);
	    break;

	 default:
	    break;
	 };
      };
   };

   if (selection==-1)
      goto exit;

   gotoparams.searchStrLen=len;
   gotoparams.dbCardNo=pfindparams->match[selmatch].dbCardNo;
   gotoparams.dbID=pfindparams->match[selmatch].dbID;
   gotoparams.recordNum=pfindparams->match[selmatch].recordNum;
   gotoparams.matchPos=pfindparams->match[selmatch].matchPos;
   gotoparams.matchFieldNum=pfindparams->match[selmatch].matchFieldNum;
   gotoparams.matchCustom=pfindparams->match[selmatch].matchCustom;
   
   LinkPublish((LinkInfoPtr)FindLinkNew(pfindparams->appCardNo,
					pfindparams->appDbID,
					&gotoparams, title));

   FrmDeleteForm(findresults);
   MemHandleUnlock(hfindparams);
   MemHandleFree(hfindparams);
   return;

 exit:
   FrmEraseForm(findresults);
   FrmDeleteForm(findresults);
   FrmSetActiveForm(FrmGetFormPtr(MainForm));
   MemHandleUnlock(hfindparams);
   MemHandleFree(hfindparams);
   FrmUpdateForm(MainForm, frmRedrawUpdateCode);
}

static Boolean MainFormHandleEvent (EventPtr e)
{
   Boolean handled = false;
   FormPtr frm;
    
   CALLBACK_PROLOGUE

      switch (e->eType) {
      case frmOpenEvent:
	 MainFormInit();
	 frm = FrmGetActiveForm();
	 FrmDrawForm(frm);
	 FrmSetFocus(frm, FrmGetObjectIndex(frm, MainField));
	 handled = true;
	 break;

      case frmCloseEvent:
	 CleanUp();
	 break;

      case menuEvent:
	 MenuEraseStatus(NULL);

	 switch(e->data.menu.itemID) {
	 }

	 handled = true;
	 break;

      case ctlSelectEvent:
	 frm=FrmGetActiveForm();
	 switch(e->data.ctlSelect.controlID) {
	 case MainButton:
	    if (FldGetTextPtr(FrmGetObjectPtr(frm, 
					      FrmGetObjectIndex(frm, 
						                MainField)))) 
	    {
	       Publish();
	       handled=true;
	    };
	    break;
	 }
	 break;

      default:
	 break;
      }

   CALLBACK_EPILOGUE

      return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
   FormPtr frm;
   Word    formId;
   Boolean handled = false;

   if (e->eType == frmLoadEvent) {
      formId = e->data.frmLoad.formID;
      frm = FrmInitForm(formId);
      FrmSetActiveForm(frm);

      switch(formId) {
      case MainForm:
	 FrmSetEventHandler(frm, MainFormHandleEvent);
	 break;
      }
      handled = true;
   }

   return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(void)
{
   FrmGotoForm(MainForm);
   return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
   FrmSaveAllForms();
   FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
   Word err;
   EventType e;

   do {
      EvtGetEvent(&e, evtWaitForever);
      if (! SysHandleEvent (&e))
	 if (! MenuHandleEvent (NULL, &e, &err))
	    if (! ApplicationHandleEvent (&e))
	       FrmDispatchEvent (&e);
   } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
   Word err;

   switch (cmd) {
   case sysAppLaunchCmdNormalLaunch:
   case linkAppLaunchRequestLink:
      err = StartApplication();
      if (err) return err;
      EventLoop();
      StopApplication();
      break;

   case linkAppLaunchFollowLinkSubCall: {
      FindLinkInfoPtr linkinfo=(FindLinkInfoPtr)cmdPBP;
      GoToParamsPtr gotoparams;

      gotoparams=MemPtrNew(sizeof(GoToParamsType));
      MemMove(gotoparams, &linkinfo->gotoparams, sizeof(GoToParamsType));
      MemPtrSetOwner(gotoparams, 0);
      SysUIAppSwitch(linkinfo->cardno, linkinfo->dbid,
		     sysAppLaunchCmdGoTo, (Ptr)gotoparams);
       
      break;
   };

   default:
      return sysErrParamErr;
   };

   return 0;
}
